@@ -25,6 +25,7 @@ DEALINGS IN THE SOFTWARE.
 """
 
 import colorsys
+import random
 
 class Colour:
     """Represents a Discord role colour. This class is similar
@@ -117,6 +118,19 @@ class Colour:
         """A factory method that returns a :class:`Colour` with a value of ``0``."""
         return cls(0)
 
+    @classmethod
+    def random(cls):
+        """A factory method that returns a :class:`Colour` with a random hue.
+
+        .. note::
+
+            The random algorithm works by choosing a colour with a random hue but
+            with maxed out saturation and value.
+
+        .. versionadded:: 1.6
+        """
+        return cls.from_hsv(random.random(), 1, 1)
+
     @classmethod
     def teal(cls):
         """A factory method that returns a :class:`Colour` with a value of ``0x1abc9c``."""
@@ -239,7 +253,7 @@ class Colour:
     def dark_theme(cls):
         """A factory method that returns a :class:`Colour` with a value of ``0x36393F``.
         This will appear transparent on Discord's dark theme.
-        
+
         .. versionadded:: 1.5
         """
         return cls(0x36393F)
